@@ -1,10 +1,16 @@
+import os
 import sys
 import json
 
 try:
     from shutil import which
 except ImportError:
-    from distutils.spawn import find_executable as which
+    from distutils.spawn import find_executable
+
+    def which(cmd, mode=os.X_OK, path=None):
+        cmd = find_executable(cmd, path)
+        if cmd and os.access(cmd, mode):
+            return cmd
 
 
 if sys.version_info > (3,):
@@ -21,8 +27,7 @@ def to_bytes(s):
     return s
 
 def json_encoder_fallback(o):
-    # Allow bytes (python3)
-    if isinstance(o, bytes):
+    if isinstance(o, (bytes, bytearray)):
         return to_unicode(o)
     return json.JSONEncoder.default(json_encoder, o)
 
@@ -45,3 +50,14 @@ json_encoder_ensure_ascii = json.JSONEncoder(
     separators=None,
     default=None,
 )
+
+def lockmethod(func):
+    def newfunc(self, *args, **kwargs):
+        if self._lock is None:
+            return func(self, *args, **kwargs)
+        self._lock.acquire()
+        try:
+            return func(self, *args, **kwargs)
+        finally:
+            self._lock.release()
+    return newfunc
